// ***********************************************************
// Project: lcd HD44780 4-bit
// Author: 
// : 1.2 
//    bv (rw     )
// ***********************************************************
#include <avr/io.h>              // Most basic include files
#include <avr/interrupt.h>       // Add the necessary ones
#include <util/delay.h> /*F_CPU = 4000000; _delay_ms max = 65,535ms; _delay_us max = 192us */


//  PC1,PC2,PC3,PC4
//  PD5(e),PD6(rs)	

#define SetBit(Port,bit) Port |= ( 0x01 << bit )
#define ClrBit(Port,bit) Port &= ( ~( 0x01 << bit ) )

#define PIN_E	7
#define SIG_E_1		PORTD|=_BV(PIN_E)
#define SIG_E_0		PORTD&=~_BV(PIN_E)
#define INIT_DDR_SIG_E	DDRB|=_BV(PIN_E)

#define PIN_RS	0
#define SIG_RS_1	PORTB|=_BV(PIN_RS)
#define SIG_RS_0	PORTB&=~_BV(PIN_RS)
#define INIT_DDR_SIG_RS	DDRB|=_BV(PIN_RS)

#define DB7	_BV(6)	//portD
#define DB6	_BV(5)	//portD
#define DB5	_BV(7)	//portB
#define DB4	_BV(6)	//portB

#define LCD_PIN_BUS (DB7|DB6|DB5|DB4);


//  lcd  !    
#define busy_flag					0x80
#define clear_display				0x01 //1.5mS  .       
#define return_home					0x02 //40uS     
//---------Entry mode set--------------- //40uS        
#define	inc_AC						0x06
//#define inc_AC_display_shift_richt	0x05
#define dec_AC						0x04
//#define dec_AC_display_shift_left	0x07
//---------Display control--------------
#define display_off					0x08
#define display_on					0x0C
#define cursor_off					0x0C		
#define cursor_lines				0x0E
#define cursor_lin_blinks			0x0F 
#define cursor_blinks				0x0D 
//---------Cursor/display shift---------
#define shift_cursor_left			0x10
#define shift_cursor_richt			0x14
#define shift_display_left			0x18
#define shift_display_richt			0x1C
//--------------Function set------------
//--------------------------------------

#define set_cgram_address				0x40 //max 63
#define set_ddram_address				0x80 //max 127


// E RW    = 0



void lcd_4bus_io(uint8_t data){
	PORTB&=~DB4;
	PORTB&=~DB5;
	PORTD&=~DB6;
	PORTD&=~DB7;
	
	//  
	if (data&_BV(7)) PORTD|=DB7;
	if (data&_BV(6)) PORTD|=DB6;
	if (data&_BV(5)) PORTB|=DB5;
	if (data&_BV(4)) PORTB|=DB4;
	SIG_E_1;
	_delay_us(3);//5us
	SIG_E_0;
}

void lcd_4bus_write(uint8_t data)	//   4  
{	
	DDRB|=DB4;									//  
	DDRB|=DB5;
	DDRD|=DB6;
	DDRD|=DB7;
	
	lcd_4bus_io(data);
	lcd_4bus_io(data<<4);
/*	
	PORTLCD&=~DB4;
	PORTLCD&=~DB5;
	PORTLCD&=~DB6;
	PORTLCD&=~DB7;
	DDRLCD&=~DB4;									//  
	DDRLCD&=~DB5;
	DDRLCD&=~DB6;
	DDRLCD&=~DB7;
//*/
}

void lcd_write_data(uint8_t data){
	//write data to CGRAM or DDRAM
	_delay_us(45);//40us
	SIG_RS_1;//RS = 1 -   (DR). RW = 0 - 
	lcd_4bus_write(data);
	SIG_RS_0;
}



void lcd_instructions_set(uint8_t address){
	//   -> address|( , cgram  ddram)
	_delay_us(45);//40us
	lcd_4bus_write(address);
}

void lcd_set_address(uint8_t x, uint8_t y){
	x|=set_ddram_address;
	if (y) x|=0x40;
	lcd_instructions_set(x);
}

void lcd_init(void){
	
	INIT_DDR_SIG_E;//     
	INIT_DDR_SIG_RS;
	SIG_E_0;
	SIG_RS_0;//0x30 5m 0x30 0x30
	
	ClrBit(PORTD,5);ClrBit(PORTD,6);
	ClrBit(PORTB,6);ClrBit(PORTB,7);				//  
//	DDRLCD=0x1F;									//   DDRLCD|=~3;
	
	_delay_ms(80);					//   20 (   5)
/*	
	PORTB|=DB5;
	SIG_E_1;
	_delay_us(3);//5us
	SIG_E_0;
	
	_delay_ms(5);
	
	SIG_E_1;
	_delay_us(3);//5us
	SIG_E_0;
	
	PORTB&=~DB5;
	PORTB|=DB7|DB6;
	
	SIG_E_1;
	_delay_us(3);//5us
	SIG_E_0;
	
	_delay_ms(5);
	
	PORTB&=~(DB7|DB6);
	*/
	
//*
	PORTB|=DB5|DB4;				//function set
	SIG_E_1;
	_delay_us(3);//5us
	SIG_E_0;
		
	_delay_ms(5); 					//2ms
	
	
	//PORTB|=DB5|DB4;				//function set
	SIG_E_1;
	_delay_us(3);//5us
	SIG_E_0;
	
	_delay_us(150);					//40us
	
	PORTB|=DB5|DB4;				//function set
	SIG_E_1;
	_delay_us(3);//5us
	SIG_E_0;
	_delay_us(150);					//40us
	
	ClrBit(PORTD,5);ClrBit(PORTD,6);
	ClrBit(PORTB,6);ClrBit(PORTB,7);
	
	PORTB|=DB5;					//function set 4bit
	SIG_E_1;
	_delay_us(3);//5us
	SIG_E_0;
	
	_delay_ms(5); 					//2ms//_delay_us(50);//40us
	lcd_instructions_set(0x2A);
	_delay_ms(5); 					//2ms	//_delay_us(50);//40us
	lcd_instructions_set(0x08);
	_delay_ms(5); 					//2ms	//_delay_us(50);//40us
	lcd_instructions_set(0x01);
	_delay_ms(5);					 //2ms	//_delay_us(50);//40us
	lcd_instructions_set(0x06);
	// Initialization ends
	
	//_delay_us(50);//40us
	lcd_instructions_set(cursor_off);
//*/
}
